package xyz.lychee.lagfixer.hooks;

import me.clip.placeholderapi.PlaceholderAPI;
import me.clip.placeholderapi.expansion.PlaceholderExpansion;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.managers.HookManager;
import xyz.lychee.lagfixer.managers.ModuleManager;
import xyz.lychee.lagfixer.managers.SupportManager;
import xyz.lychee.lagfixer.modules.WorldCleanerModule;
import xyz.lychee.lagfixer.objects.AbstractHook;
import xyz.lychee.lagfixer.objects.AbstractMonitor;

public class PlaceholderAPIHook extends AbstractHook {
   private static PlaceholderAPIHook instance;
   private PapiImplementation papi;

   public PlaceholderAPIHook(LagFixer plugin, HookManager manager) {
      super(plugin, manager);
      instance = this;
   }

   public String applyPlaceholders(Player p, String text) {
      return PlaceholderAPI.setPlaceholders(p, text);
   }

   public void load() {
      this.papi = new PapiImplementation(this.getPlugin());
      this.papi.register();
   }

   public void disable() {
      this.papi.unregister();
   }

   public String plugin() {
      return "PlaceholderAPI";
   }

   public PapiImplementation getPapi() {
      return this.papi;
   }

   public static PlaceholderAPIHook getInstance() {
      return instance;
   }

   public static class PapiImplementation extends PlaceholderExpansion {
      private final LagFixer plugin;

      public PapiImplementation(LagFixer plugin) {
         this.plugin = plugin;
      }

      public @NotNull String getIdentifier() {
         return "lagfixer";
      }

      public @NotNull String getAuthor() {
         return "lychee";
      }

      public @NotNull String getVersion() {
         return this.plugin.getDescription().getVersion();
      }

      public String onPlaceholderRequest(Player p, @NotNull String id) {
         return this.response(id);
      }

      public String onRequest(OfflinePlayer p, @NotNull String id) {
         return this.response(id);
      }

      public boolean persist() {
         return true;
      }

      public boolean canRegister() {
         return true;
      }

      public String response(String id) {
         AbstractMonitor monitor = SupportManager.getInstance().getMonitor();
         switch (id.toLowerCase()) {
            case "tps":
               return Double.toString(monitor.getTps());
            case "mspt":
               return Double.toString(monitor.getMspt());
            case "cpuprocess":
               return Double.toString(monitor.getCpuProcess());
            case "cpusystem":
               return Double.toString(monitor.getCpuSystem());
            case "worldcleaner":
               WorldCleanerModule module = (WorldCleanerModule)ModuleManager.getInstance().get(WorldCleanerModule.class);
               return module != null && module.isLoaded() ? module.getSecond() + "s" : null;
            default:
               return null;
         }
      }
   }
}
